#define STRICT						// Strict controls
#include <Windows.h>
#include <Windowsx.h>
#include <Commdlg.h>
#include <stdlib.h>
#include <math.h>

#include "OleIncl.h"
#include "Incline.h"

//----------------------------------------------------------------
//							Declarations
//----------------------------------------------------------------

// ---- External functions from OLEPROC.C
BOOL MyOleInit (HINSTANCE, LPSTR, WORD);
BOOL MySaveDoc (VOID);
VOID MyRevoke (VOID);
VOID MyWaitShow (VOID);

// ---- Global declarations
#define			TOPCLASS	"Inclined"
#define			ABS(D)		(D<0?-D:D)	// For doubles

// ---- Next structure defines instance variables
typedef struct	_TXTDATA
{
	INCLINEDTEXT	ti;				// Text data (defined in .h file)
	HFONT		hfont;              // Font
	int			var;				// Angle change variation
}
				TXTDATA, FAR *LPTXTDATA;

// ---- Instance variable
TXTDATA			td;

//----------------------------------------------------------------
//							Common functions
//----------------------------------------------------------------

//                           ----------------
// -------------------------    GetTextBox    ---------------
//                           ----------------

// This function fills the SIZE structure with the dimensions
// of the surrounding rectangle for the text szTxt whose length is
// lenTxt in the hfont font, with the angle nAngle, in the window
// hwnd. See text for a comparision with GetTextExtent

VOID GetTextBox (HWND hwnd, HFONT hfont, int nAngle, LPSTR szTxt,
				UINT lenTxt, LPSIZE lpsize, LPPOINT lppt)
{
HDC				hdc;
HFONT			hfontOld;
double			dAngle;
DWORD			dwExt;
double			w, h;				// Text dimensions

	hdc = GetDC (hwnd);
	if (hfont)
		hfontOld = SelectFont (hdc, hfont);
	dwExt = GetTextExtent (hdc, szTxt, lenTxt);
	if (hfont)
		SelectFont (hdc, hfontOld);
	ReleaseDC (hwnd, hdc);
	w = LOWORD (dwExt);
	h = HIWORD (dwExt);

// ---- Calculates angle
	while (nAngle < 0)
		nAngle += 3600;				// Makes it positive
	nAngle %= 3600;					// Between 0 and 3600
	dAngle = ((double)nAngle*atan(1)*4)/1800;	// In radians and doubles

// ---- Calculates rectangle
	lpsize->cx = (int)(ABS (w * cos (dAngle)) + ABS (h * sin (dAngle)));
	lpsize->cy = (int)(ABS (w * sin (dAngle)) + ABS (h * cos (dAngle)));

// ---- Calculates origine
	if (nAngle < 900)
	{
		lppt->x = 0;
		lppt->y = (int)(ABS (w * sin (dAngle)));
	}
	else if (nAngle < 1800)
	{
		lppt->x = (int)(ABS (w * cos (dAngle)));
		lppt->y = lpsize->cy;
	}
	else if (nAngle < 2700)
	{
		lppt->x = lpsize->cx;
		lppt->y = (int)(ABS (h * sin (dAngle-900)));
	}
	else
	{
		lppt->x = (int)(ABS (h * cos (dAngle-900)));
		lppt->y = 0;
	}
}

//                           ----------------
// -------------------------    MyDrawText    ---------------
//                           ----------------

VOID MyDrawText (HDC hdc, POINT pt)
{
HFONT			hfontOld;

	hfontOld = SelectFont (hdc, td.hfont);
	SetTextColor (hdc, td.ti.clrText);

	TextOut (hdc, pt.x, pt.y, td.ti.szTxt, lstrlen (td.ti.szTxt));
	SelectFont (hdc, hfontOld);
}

//                           ----------------
// -------------------------   MyChangeFont   ---------------
//                           ----------------

VOID MyChangeFont (HWND hwnd)
{
BOOL			fTrans;
int				nAngle;
HDC				hdc;

	if (td.hfont)
		DeleteFont (td.hfont);

	nAngle = GetDlgItemInt (hwnd, IDD_TXTANGLE, &fTrans, TRUE);
	if (!fTrans)
		nAngle = 0;
	td.ti.lf.lfEscapement = nAngle;

	td.hfont = CreateFontIndirect (&td.ti.lf);
	InvalidateRect (GetDlgItem (hwnd, IDD_PREVIEW), NULL, TRUE);

	SetDlgItemText (hwnd, IDD_FONTNAME, (LPSTR)td.ti.lf.lfFaceName);
	hdc = GetDC (hwnd);
	SetDlgItemInt (hwnd, IDD_FONTSIZE,
		abs (MulDiv (td.ti.lf.lfHeight, 72,
			GetDeviceCaps(hdc, LOGPIXELSY))), FALSE);
	ReleaseDC (hwnd, hdc);
}

//                           ---------------------
// ------------------------- ConvertRectToHiMetric ---------------
//                           ---------------------

void FAR PASCAL ConvertRectToHiMetric (HWND hwnd, LPRECT lprc)
{
HDC				hdc;
POINT			pt;

	hdc = GetDC (hwnd);
	SetMapMode (hdc, MM_HIMETRIC);

	SetWindowOrg (hdc, 0, 0);
	SetWindowExt (hdc, lprc->right, lprc->bottom);

//---- Converts top left corner
	pt.x = lprc->left;
	pt.y = lprc->top;
	DPtoLP (hdc, &pt, 1);
	lprc->left = pt.x;
	lprc->top = -pt.y;

//---- Converts bottom right corner
	pt.x = lprc->right;
	pt.y = lprc->bottom;
	DPtoLP (hdc, &pt, 1);
	lprc->right = pt.x;
	lprc->bottom = -pt.y;

	ReleaseDC (hwnd, hdc);
	return;
}

//                           ---------------------
// -------------------------      MakeNative      ---------------
//                           ---------------------

HANDLE MakeNative (HWND hwnd)
{
HANDLE			hmem;
LPINCLINEDTEXT	lpti;

// ---- Returns a "Native" format memory block
		hmem = GlobalAlloc (GHND | GMEM_DDESHARE, sizeof (INCLINEDTEXT));
		lpti = (LPINCLINEDTEXT)GlobalLock (hmem);
		if (!lpti)
		{
			GlobalFree (hmem);
			return NULL;			// Err0r
		}

		*lpti = td.ti;				// Stores data
		GlobalUnlock (hmem);

		return hmem;
}

//                           ---------------------
// -------------------------    MakeOwnerLink     ---------------
//                           ---------------------

HANDLE MakeOwnerLink (HWND hwnd)
{
HANDLE			hmem;
LPSTR			lpsz;
char			szBuf[80];

		LoadString (GetWindowInstance (hwnd), IDS_KEY_SERVER, szBuf, sizeof szBuf);
		hmem = GlobalAlloc (GHND | GMEM_DDESHARE, lstrlen (szBuf)+9);
		lpsz = GlobalLock (hmem);
		if (!lpsz)
		{
			GlobalFree (hmem);
			return NULL;			// Error
		}

		lstrcpy (lpsz, szBuf);		// Object class name
		lpsz += (lstrlen (lpsz) + 1);
		*lpsz++ = 0;				// Document : nul
		lstrcpy (lpsz, "Texte");	// Object individual name
		lpsz += (lstrlen(lpsz) + 1);
		*lpsz = 0;					// Final 0
		GlobalUnlock (hmem);

		return hmem;
}

//                           ---------------------
// -------------------------   MakeMetaFilePict    ---------------
//                           ---------------------

HANDLE MakeMetaFilePict (HWND hwnd)
{
HANDLE			hmem;
HDC				hdc;
HANDLE			hMF;
LPMETAFILEPICT	lpmfp;
SIZE			size;
POINT			pt;
RECT			rc;

// ---- Creates metafile
		hdc = CreateMetaFile (NULL);

		GetTextBox (hwnd, td.hfont, td.ti.lf.lfEscapement, td.ti.szTxt,
					lstrlen (td.ti.szTxt), &size, &pt);

		SetMapMode (hdc, MM_ANISOTROPIC);
		SetWindowOrg(hdc, 0, 0);
		SetWindowExt(hdc, size.cx, size.cy);

		MyDrawText (hdc, pt);

		hMF = CloseMetaFile (hdc);

// ---- Creates METAFILEPICT memory block
		hmem = GlobalAlloc (GHND | GMEM_DDESHARE, sizeof (METAFILEPICT));
		lpmfp = GlobalLock (hmem);
		if (!lpmfp)
		{
			GlobalFree (hmem);
			return NULL;			// Error
		}

		lpmfp->mm = MM_ANISOTROPIC;
		rc.left = rc.top = 0;
		rc.right = size.cx;
		rc.bottom = size.cy;
		ConvertRectToHiMetric (hwnd, &rc);
		lpmfp->xExt = rc.right;
		lpmfp->yExt = rc.bottom;
		lpmfp->hMF = hMF;
		GlobalUnlock (hmem);

		return hmem;
}

//                           ---------------------
// -------------------------      MakeBitmap      ---------------
//                           ---------------------

HBITMAP MakeBitmap (HWND hwnd)
{
HDC				hdc, hdcMem;
HBITMAP			hbmp, hbmpOld;
SIZE			size;
RECT			rc;
POINT			pt;

// ---- Creates bitmap
		hdc = GetDC (hwnd);
		hdcMem = CreateCompatibleDC (hdc);
		hbmp = CreateCompatibleBitmap (hdc, size.cx, size.cy);
		if (hbmp)
		{
			hbmpOld = SelectBitmap (hdcMem, hbmp);
			rc.left = rc.top = 0;
			rc.right = size.cx;
			rc.bottom = size.cy;
			FillRect (hdcMem, &rc, GetStockBrush (WHITE_BRUSH));
			MyDrawText (hdcMem, pt);
			SelectBitmap (hdcMem, hbmpOld);
		}
		DeleteDC (hdcMem);
		ReleaseDC (hwnd, hdc);

		return hbmp;
}

//                           ---------------------
// -------------------------        PutsData       ---------------
//                           ---------------------

VOID PutsData (LPINCLINEDTEXT lpti)
{
	td.ti = *lpti;

	SetDlgItemText (AppGbl.hwnd, IDD_EDIT, td.ti.szTxt);
	SetDlgItemInt (AppGbl.hwnd, IDD_TXTANGLE, td.ti.lf.lfEscapement, TRUE);
	MyChangeFont (AppGbl.hwnd);
}

//----------------------------------------------------------------
//						Message handling functions
//----------------------------------------------------------------

//                           ----------------
// ------------------------- Top_OnInitDialog ---------------
//                           ----------------

BOOL Top_OnInitDialog (HWND hwnd, HWND hwndFocus, DWORD dwData)
{
// ---- Stores handle
	AppGbl.hwnd = hwnd;

// ---- Initializes
/*********** Only if standalone ***************
	MyChangeFont (hwnd);
	SetDlgItemText (hwnd, IDD_EDIT, td.ti.szTxt);
	SetDlgItemInt (hwnd, IDD_TXTANGLE, td.ti.lf.lfEscapement, TRUE);
**************************************************/
	Edit_LimitText (GetDlgItem (hwnd, IDD_EDIT), MAXTXT);
	CheckRadioButton (hwnd, IDD_1, IDD_100, IDD_100);
	td.var = 100;

// ---- Starts font dialogbox
	MyWaitShow ();
	if (td.ti.lf.lfHeight == 0)
	{
		PostMessage (hwnd, WM_COMMAND, IDD_FONT, 0);
		return FALSE;
	}

	return TRUE;
}

//                           ----------------
// -------------------------   Top_OnDestroy  ---------------
//                           ----------------

VOID Top_OnDestroy (HWND hwnd)
{
// ---- Cleans up
	if (td.hfont)
		DeleteFont (td.hfont);
}

//                           -------------
// ------------------------- Top_OnCommand ---------------
//                           -------------

VOID Top_OnCommand (HWND hwnd, int id, HWND hwndCtl, WORD codeNotify)
{
	switch (id)
	{
// -------- OK button --------
	case IDOK:
// ---- Updates
		SendMessage (hwnd, WM_COMMAND, IDD_APPLY, 0);
// ---- !!!!! carries on !!!!!

// -------- CANCEL cancel --------
	case IDCANCEL:
// ---- Ends
		MyRevoke ();
		EndDialog (hwnd, 0);
		break;

// -------- Change in text --------
	case IDD_EDIT:
		if (codeNotify == EN_CHANGE)
		{
			GetDlgItemText (hwnd, IDD_EDIT, td.ti.szTxt, MAXTXT);
			InvalidateRect (GetDlgItem (hwnd, IDD_PREVIEW), NULL, TRUE);
			SetDlgItemText (hwnd, IDD_PREVIEW, td.ti.szTxt);
		}
		break;

// -------- Change in angle --------
	case IDD_TXTANGLE:
		if (codeNotify == EN_CHANGE)
			MyChangeFont (hwnd);
	    break;

// -------- Angle variation --------
	case IDD_1:
	case IDD_10:
	case IDD_100:
		td.var = id == IDD_1? 1: id == IDD_10? 10: 100;
		break;

// -------- Changement in font --------
	case IDD_FONT:
	{
	CHOOSEFONT	chf;

// ---- Calls standard dialogbox
		chf.lStructSize = sizeof (CHOOSEFONT);
		chf.hwndOwner = hwnd;
		chf.lpLogFont = &td.ti.lf;
		chf.Flags = CF_FORCEFONTEXIST | CF_SCREENFONTS | CF_SCALABLEONLY | CF_SHOWHELP | CF_EFFECTS | CF_INITTOLOGFONTSTRUCT;
		chf.rgbColors = td.ti.clrText;
		chf.nFontType = SCREEN_FONTTYPE;

		if (ChooseFont (&chf))
		{
			td.ti.clrText = chf.rgbColors;
			MyChangeFont (hwnd);
		}

		else if (*td.ti.lf.lfFaceName == 0)	// No font, ends
			PostMessage (hwnd, WM_COMMAND, IDCANCEL, 0);

		SetFocus (GetDlgItem (hwnd, IDD_EDIT));
		break;
	}

// -------- Apply --------
	case IDD_APPLY:
		if (!MySaveDoc ())
		{
		char	szBuf1[80], szBuf2[80];

			LoadString (GetWindowInstance (hwnd), IDS_ERRUPD, szBuf1, sizeof szBuf1);
			LoadString (GetWindowInstance (hwnd), IDS_TITLE, szBuf2, sizeof szBuf2);
			MessageBox (hwnd, szBuf1, szBuf2, MB_OK | MB_ICONEXCLAMATION);
		}
		SetFocus (GetDlgItem (hwnd, IDD_EDIT));
		break;
/************ Following code copies into clipboard *************
	{
	HANDLE		hmem;
	HBITMAP		hbmp;

// ---- Initialize clipboard
		OpenClipboard (hwnd);
		EmptyClipboard ();

// -------- 1 - "Native" format --------
		hmem = MakeNative (hwnd);
		if (hmem)
			SetClipboardData (OleGbl.cfNative, hmem);
		else
		{
			CloseClipboard ();
			break;					// Error
		}

// -------- 2 - "OwnerLink" format --------
		hmem = MakeOwnerLink (hwnd);
		if (hmem)
			SetClipboardData (OleGbl.cfOwnerLink, hmem);
		else
		{
			CloseClipboard ();
			break;					// Error
		}

// -------- 3 - CF_METAFILEPICT format --------
		hmem = MakeMetaFilePict (hwnd);
		if (hmem)
			SetClipboardData (CF_METAFILEPICT, hmem);
		else
		{
			CloseClipboard ();
			break;					// Error
		}

// -------- 4 - CF_BITMAP Format --------
		hbmp = MakeBitmap (hwnd);
		if (hbmp)
			SetClipboardData (CF_BITMAP, hbmp);
		else
		{
			CloseClipboard ();
			break;					// Error
		}

// ---- Closes clipboard
		CloseClipboard ();
	}
*******************************************************************/
	}

	return;
}

//							 -------------
// ------------------------- Top_OnVScroll ---------------
//							 -------------

VOID Top_OnVScroll (HWND hwnd, HWND hwndCtl, UINT code, int pos)
{
int 			n = 0, n1;
BOOL			fTrans;

	switch (code)
	{
	case SB_LINEDOWN:
		n = -td.var;
		break;

	case SB_LINEUP:
		n = td.var;
		break;
	}

	if (n)
	{
		n1 = GetDlgItemInt (hwnd, IDD_TXTANGLE, &fTrans, TRUE);
		if (!fTrans)
			n1 = 0;
		n += n1;
		SetDlgItemInt (hwnd, IDD_TXTANGLE, n, TRUE);
	}

	SetFocus (GetDlgItem (hwnd, IDD_EDIT));
	return;
}

//                           --------------
// ------------------------- Top_OnDrawItem ---------------
//                           --------------

VOID Top_OnDrawItem (HWND hwnd, const DRAWITEMSTRUCT FAR* lpdis)
{
SIZE			size;
POINT			pt;
HBRUSH			hbrOld;
HPEN			hpen, hpenOld;

	if (lpdis->CtlID == IDD_PREVIEW
			&& lpdis->itemAction & ODA_DRAWENTIRE)
	{
// ---- Draws text
		GetTextBox (hwnd, td.hfont, td.ti.lf.lfEscapement, td.ti.szTxt,
					lstrlen (td.ti.szTxt), &size, &pt);

		MyDrawText (lpdis->hDC, pt);

// ---- Adds a dotted rectangle
		hbrOld = SelectBrush (lpdis->hDC, GetStockBrush (NULL_BRUSH));
		hpen = CreatePen (PS_DOT, 1, GetSysColor (COLOR_WINDOWTEXT));
		hpenOld = SelectPen (lpdis->hDC, hpen);
		Rectangle (lpdis->hDC, 0, 0, size.cx, size.cy);
		SelectBrush (lpdis->hDC, hbrOld);
		SelectPen (lpdis->hDC, hpenOld);
		DeletePen (hpen);
	}

	return;
}

//                           ----------
// ------------------------- TopDlgProc -------------------
//                           ----------

BOOL CALLBACK _export TopDlgProc (HWND hwnd, WORD msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
	case WM_INITDIALOG:
		return (BOOL)HANDLE_WM_INITDIALOG (hwnd, wParam, lParam, Top_OnInitDialog);

	case WM_DESTROY:
		HANDLE_WM_DESTROY (hwnd, wParam, lParam, Top_OnDestroy);
		return TRUE;

	case WM_COMMAND:
		HANDLE_WM_COMMAND (hwnd, wParam, lParam, Top_OnCommand);
		return TRUE;

	case WM_VSCROLL:
		HANDLE_WM_VSCROLL (hwnd, wParam, lParam, Top_OnVScroll);
		return TRUE;

	case WM_DRAWITEM:
		HANDLE_WM_DRAWITEM (hwnd, wParam, lParam, Top_OnDrawItem);
		return TRUE;

	default:
		return FALSE;
	}
}

//							 -------
// ------------------------- WinMain -------------------
//                           -------

int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
					LPSTR lpCmdLine, int nCmdShow)
{
DLGPROC 		lpDlgProc;

// ---- OLE initializations
	if (!MyOleInit (hInstance, lpCmdLine, nCmdShow))
		return 1;					// Error

// ---- Creates dialogbox
	lpDlgProc = (DLGPROC)MakeProcInstance ((FARPROC)TopDlgProc, hInstance);
	DialogBox (hInstance, MAKEINTRESOURCE (100), NULL, lpDlgProc);
	FreeProcInstance ((FARPROC)lpDlgProc);

	return 0;
}
